@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -29,6 +29,7 @@ use Xibo\Entity\Widget;
 use Xibo\Event\MediaDeleteEvent;
 use Xibo\Event\WidgetAddEvent;
 use Xibo\Event\WidgetEditEvent;
+use Xibo\Event\WidgetEditOptionRequestEvent;
 use Xibo\Factory\DataSetFactory;
 use Xibo\Factory\DisplayFactory;
 use Xibo\Factory\DisplayGroupFactory;
@@ -784,6 +785,12 @@ class Module extends Base
             $templates = $module->templatesAvailable(true);
         }
 
+        // do we have special options requested?
+        // Dashboards Widget needs to pull available services from connector
+        $event = new WidgetEditOptionRequestEvent($module->widget);
+        $this->getDispatcher()->dispatch($event, $event::$NAME);
+        $options = $event->getOptions();
+
         // Pass to view
         $this->getState()->template = $module->editForm($request);
         $this->getState()->setData($module->setTemplateData([
@@ -791,7 +798,8 @@ class Module extends Base
             'media' => $media,
             'validExtensions' => str_replace(',', '|', $module->getModule()->validExtensions),
             'templatesAvailable' => $templates,
-            'isTopLevel' => $this->playlistFactory->getById($module->widget->playlistId)->isRegionPlaylist()
+            'options' => $options,
+            'isTopLevel' => $this->playlistFactory->getById($module->widget->playlistId)->isRegionPlaylist(),
         ]));
 
         return $this->render($request, $response);
