@@ -104,7 +104,8 @@ class SavedReport extends Base
             'useRegexForName' => $sanitizedQueryParams->getCheckbox('useRegexForName'),
             'userId' => $sanitizedQueryParams->getInt('userId'),
             'reportName' => $sanitizedQueryParams->getString('reportName'),
-            'onlyMyReport' => $sanitizedQueryParams->getCheckbox('onlyMyReport')
+            'onlyMyReport' => $sanitizedQueryParams->getCheckbox('onlyMyReport'),
+            'logicalOperatorName' => $sanitizedQueryParams->getString('logicalOperatorName'),
         ], $sanitizedQueryParams));
 
         foreach ($savedReports as $savedReport) {
@@ -311,12 +312,7 @@ class SavedReport extends Base
         // Set Template
         $this->getState()->template = $this->reportService->getSavedReportTemplate($name);
 
-        $this->getState()->setData([
-            'metadata' => $results->metadata,
-            'table' => $results->table,
-            'chart' => $results->chart,
-            'hasChartData' => $results->hasChartData,
-        ]);
+        $this->getState()->setData($results->jsonSerialize());
 
         return $this->render($request, $response);
     }
